using System;
using System.Data;
using Microsoft.SqlServer.Server;
using System.Data.SqlTypes;
using System.IO;
using System.Text;
using System.Collections.Generic;

//Needs to pass binary data to SQL
[Serializable]
//Defines the contract for Init, Accumilate, Merge and Terminate
[SqlUserDefinedAggregate(Format.UserDefined, MaxByteSize = 8000)]
public class Median : IBinarySerialize
{
    private List<SqlDouble> AccumilatedData;
    private SqlDouble Result;

    public void Init() //Initialize call to aggregate
    {
        AccumilatedData = new List<SqlDouble>();
        Result = 0;
    }

    //Called on every row that is passed to the aggregate
    public void Accumulate(SqlDouble value)
    {
        if (value.IsNull)
        {
            return;
        }

        //Accumilate each row to a list
        AccumilatedData.Add(value.Value);
    }

    //Called if another type of Median is merged with this one
    public void Merge(Median other)
    {
        AccumilatedData.AddRange(other.AccumilatedData);
    }

    //Called at the end of the aggregate to return the result
    public SqlDouble Terminate()
    {
        return Result;
    }

    //Called before terminate to read the temporarily stored data back to the aggregate
    public void Read(BinaryReader r)
    {
        Result = r.ReadDouble();
    }

    //Writes the data to temporary storage
    public void Write(BinaryWriter w)
    {
        w.Write((Double)GetMedian(AccumilatedData));
    }

    public SqlDouble GetMedian(List<SqlDouble> NumericCollection)
    {
        Int32 CollectionCount = NumericCollection.Count;
        //Can't use lambda expressions or IEnumerable<t>; due to the lack of the System.Linq assembly
        //Can't use anonymous delegates either
        NumericCollection.Sort(new Comparison<SqlDouble>(SortValues));

        if (CollectionCount > 100) //Trial Version
            return 0;

        if (CollectionCount % 2 == 0)
        {
            //Collection count is even so get the two middle values and divide by 2
            SqlDouble a = NumericCollection[(CollectionCount / 2) - 1];
            SqlDouble b = NumericCollection[(CollectionCount / 2)];
            return (a + b) / 2;
        }
        else
        {
            //Get the middle value
            return NumericCollection[CollectionCount / 2];
        }
    }

    public int SortValues(SqlDouble a, SqlDouble b)
    {
        //Short hand if PercentileCalculatorement to determine if 'a' is less then/equal to or greater than 'b'
        return a < b ? (a == b ? 0 : -1) : 1;
    }
}